@@ -1,82 +1,94 @@
-wetherill <- function(x,i){
+wetherill <- function(x,i=1,format){
+    if (missing(format)){
+        X <- x$x
+        format <- x$format
+    } else {
+        X <- x
+    }
     out <- list()
-    if (x$format < 4) labels <- c('Pb207U235','Pb206U238')
-    else if (x$format < 7) labels <- c('Pb207U235','Pb206U238','Pb204U238')
+    if (format < 4) labels <- c('Pb207U235','Pb206U238')
+    else if (format < 7) labels <- c('Pb207U235','Pb206U238','Pb204U238')
     else labels <- c('Pb207U235','Pb206U238','Pb208Th232','Th232U238')
-    if (x$format %in% c(1,3)){
-        out$x <- x$x[i,labels]
-        out$cov <- cor2cov2(x$x[i,'errPb207U235'],x$x[i,'errPb206U238'],x$x[i,'rhoXY'])
-    } else if (x$format == 2){
+    if (format %in% c(1,3)){
+        out$x <- X[i,labels]
+        out$cov <- cor2cov2(X[i,'errPb207U235'],X[i,'errPb206U238'],X[i,'rhoXY'])
+    } else if (format == 2){
         U238U235 <- iratio('U238U235')[1]
-        Pb207U235 <- U238U235*x$x[i,'Pb207Pb206']/x$x[i,'U238Pb206']
-        Pb206U238 <- 1/x$x[i,'U238Pb206']
+        Pb207U235 <- U238U235*X[i,'Pb207Pb206']/X[i,'U238Pb206']
+        Pb206U238 <- 1/X[i,'U238Pb206']
         out$x <- c(Pb207U235,Pb206U238)
         J <- matrix(0,2,2)
-        J[1,1] <- -Pb207U235/x$x[i,'U238Pb206']
-        J[1,2] <- U238U235/x$x[i,'U238Pb206']
-        J[2,1] <- -1/x$x[i,'U238Pb206']^2
-        E <- cor2cov2(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],x$x[i,'rhoXY'])
+        J[1,1] <- -Pb207U235/X[i,'U238Pb206']
+        J[1,2] <- U238U235/X[i,'U238Pb206']
+        J[2,1] <- -1/X[i,'U238Pb206']^2
+        E <- cor2cov2(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],X[i,'rhoXY'])
         out$cov <- J %*% E %*% t(J)
-    } else if (x$format == 4){
-        out$x <- x$x[i,labels]
-        out$cov <- cor2cov3(x$x[i,'errPb207U235'],x$x[i,'errPb206U238'],
-                            x$x[i,'errPb204U238'],x$x[i,'rhoXY'],
-                            x$x[i,'rhoXZ'],x$x[i,'rhoYZ'])
-    } else if (x$format == 5){
+    } else if (format == 4){
+        out$x <- X[i,labels]
+        out$cov <- cor2cov3(X[i,'errPb207U235'],X[i,'errPb206U238'],
+                            X[i,'errPb204U238'],X[i,'rhoXY'],
+                            X[i,'rhoXZ'],X[i,'rhoYZ'])
+    } else if (format == 5){
         U238U235 <- iratio('U238U235')[1]
-        Pb207U235 <- U238U235*x$x[i,'Pb207Pb206']/x$x[i,'U238Pb206']
-        Pb206U238 <- 1/x$x[i,'U238Pb206']
-        Pb204U238 <- x$x[i,'Pb204Pb206']/x$x[i,'U238Pb206']
+        Pb207U235 <- U238U235*X[i,'Pb207Pb206']/X[i,'U238Pb206']
+        Pb206U238 <- 1/X[i,'U238Pb206']
+        Pb204U238 <- X[i,'Pb204Pb206']/X[i,'U238Pb206']
         out$x <- c(Pb207U235,Pb206U238,Pb204U238)
         J <- matrix(0,3,3)
-        J[1,1] <- -Pb207U235/x$x[i,'U238Pb206']
-        J[1,2] <- U238U235/x$x[i,'U238Pb206']
-        J[2,1] <- -Pb206U238/x$x[i,'U238Pb206']
-        J[3,1] <- -Pb204U238/x$x[i,'U238Pb206']
-        J[3,3] <- 1/x$x[i,'U238Pb206']
-        E <- cor2cov3(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],
-                      x$x[i,'errPb204Pb206'],x$x[i,'rhoXY'],
-                      x$x[i,'rhoXZ'],x$x[i,'rhoYZ'])
+        J[1,1] <- -Pb207U235/X[i,'U238Pb206']
+        J[1,2] <- U238U235/X[i,'U238Pb206']
+        J[2,1] <- -Pb206U238/X[i,'U238Pb206']
+        J[3,1] <- -Pb204U238/X[i,'U238Pb206']
+        J[3,3] <- 1/X[i,'U238Pb206']
+        E <- cor2cov3(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],
+                      X[i,'errPb204Pb206'],X[i,'rhoXY'],
+                      X[i,'rhoXZ'],X[i,'rhoYZ'])
         out$cov <- J %*% E %*% t(J)        
-    } else if (x$format == 6){
-        out$x <- x$x[i,labels]
+    } else if (format == 6){
+        out$x <- X[i,labels]
         out$cov <- matrix(0,3,3)
         diag(out$cov) <-
-            x$x[i,c('errPb207U235','errPb206U238','errPb204U238')]^2
+            X[i,c('errPb207U235','errPb206U238','errPb204U238')]^2
         out$cov[1,2] <-
-            get.cov.75.68(x$x[i,'Pb207U235'],x$x[i,'errPb207U235'],
-                          x$x[i,'Pb206U238'],x$x[i,'errPb206U238'],
-                          x$x[i,'Pb207Pb206'],x$x[i,'errPb207Pb206'])
+            get.cov.75.68(X[i,'Pb207U235'],X[i,'errPb207U235'],
+                          X[i,'Pb206U238'],X[i,'errPb206U238'],
+                          X[i,'Pb207Pb206'],X[i,'errPb207Pb206'])
         out$cov[1,3] <-
-            get.cov.75.48(x$x[i,'Pb207U235'],x$x[i,'errPb207U235'],
-                          x$x[i,'Pb204U238'],x$x[i,'errPb204U238'],
-                          x$x[i,'Pb204Pb207'],x$x[i,'errPb204Pb207'])
+            get.cov.75.48(X[i,'Pb207U235'],X[i,'errPb207U235'],
+                          X[i,'Pb204U238'],X[i,'errPb204U238'],
+                          X[i,'Pb204Pb207'],X[i,'errPb204Pb207'])
         out$cov[2,3] <-
-            get.cov.68.48(x$x[i,'Pb206U238'],x$x[i,'errPb206U238'],
-                          x$x[i,'Pb204U238'],x$x[i,'errPb204U238'],
-                          x$x[i,'Pb204Pb206'],x$x[i,'errPb204Pb206'])
+            get.cov.68.48(X[i,'Pb206U238'],X[i,'errPb206U238'],
+                          X[i,'Pb204U238'],X[i,'errPb204U238'],
+                          X[i,'Pb204Pb206'],X[i,'errPb204Pb206'])
         out$cov[2,1] <- out$cov[1,2]
         out$cov[3,1] <- out$cov[1,3]
         out$cov[3,2] <- out$cov[2,3]
-    } else if (x$format == 7){
+    } else if (format == 7){
+        out$x <- X[i,labels]
+        out$cov <- cor2cov4(X[i,'errPb207U235'],X[i,'errPb206U238'],
+                            X[i,'errPb208Th232'],X[i,'errTh232U238'],
+                            X[i,'rhoXY'],X[i,'rhoXZ'],X[i,'rhoXW'],
+                            X[i,'rhoYZ'],X[i,'rhoYW'],X[i,'rhoZW'])
+    } else if (format == 8){
         U238U235 <- iratio('U238U235')[1]
-        Pb207U235 <- U238U235*x$x[i,'Pb207Pb206']/x$x[i,'U238Pb206']
-        Pb206U238 <- 1/x$x[i,'U238Pb206']
-        Pb208Th232 <- x$x[i,'Pb208Pb206']/(x$x[i,'U238Pb206']*x$x[i,'Th232U238'])
-        Th232U238 <- x$x[i,'Th232U238']
+        Pb207U235 <- U238U235*X[i,'Pb207Pb206']/X[i,'U238Pb206']
+        Pb206U238 <- 1/X[i,'U238Pb206']
+        Pb208Th232 <- X[i,'Pb208Pb206']/(X[i,'U238Pb206']*X[i,'Th232U238'])
+        Th232U238 <- X[i,'Th232U238']
         out$x <- c(Pb207U235,Pb206U238,Pb208Th232,Th232U238)
         J <- matrix(0,4,4)
-        J[1,1] <- -Pb207U235/x$x[i,'U238Pb206']
-        J[1,2] <- U238U235/x$x[i,'U238Pb206']
-        J[2,1] <- -Pb206U238/x$x[i,'U238Pb206']
-        J[3,1] <- -Pb208Th232/x$x[i,'U238Pb206']
-        J[3,3] <- 1/(x$x[i,'U238Pb206']*x$x[i,'Th232U238'])
-        J[3,4] <- -Pb208Th232/x$x[i,'Th232U238']
+        J[1,1] <- -Pb207U235/X[i,'U238Pb206']
+        J[1,2] <- U238U235/X[i,'U238Pb206']
+        J[2,1] <- -Pb206U238/X[i,'U238Pb206']
+        J[3,1] <- -Pb208Th232/X[i,'U238Pb206']
+        J[3,3] <- 1/(X[i,'U238Pb206']*X[i,'Th232U238'])
+        J[3,4] <- -Pb208Th232/X[i,'Th232U238']
         J[4,4] <- 1
-        E <- cor2cov4(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],
-                      x$x[i,'errPb208Pb206'],x$x[i,'errTh232U238'],
-                      x$x[i,'rhoXY'],x$x[i,'rhoXZ'],x$x[i,'rhoXW'],
-                      x$x[i,'rhoYZ'],x$x[i,'rhoYW'],x$x[i,'rhoZW'])
+        E <- cor2cov4(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],
+                      X[i,'errPb208Pb206'],X[i,'errTh232U238'],
+                      X[i,'rhoXY'],X[i,'rhoXZ'],X[i,'rhoXW'],
+                      X[i,'rhoYZ'],X[i,'rhoYW'],X[i,'rhoZW'])
         out$cov <- J %*% E %*% t(J)        
     }
     names(out$x) <- labels
@@ -85,85 +97,111 @@ wetherill <- function(x,i){
     class(out) <- "wetherill"
     out
 }
-tera.wasserburg <- function(x,i){
+tera.wasserburg <- function(x,i=1,format){
+    if (missing(format)){
+        X <- x$x
+        format <- x$format
+    } else {
+        X <- x
+    }
     out <- list()
-    if (x$format < 4) labels <- c('U238Pb206','Pb207Pb206')
-    else if (x$format < 7) labels <- c('U238Pb206','Pb207Pb206','Pb204Pb206')
+    if (format < 4) labels <- c('U238Pb206','Pb207Pb206')
+    else if (format < 7) labels <- c('U238Pb206','Pb207Pb206','Pb204Pb206')
     else labels <- c('U238Pb206','Pb207Pb206','Pb208Pb206','Th232U238')
-    if (x$format==1){
+    if (format==1){
         U238U235 <- iratio('U238U235')[1]
-        U238Pb206 <- 1/x$x[i,'Pb206U238']
-        Pb207Pb206 <- x$x[i,'Pb207U235']/(x$x[i,'Pb206U238']*U238U235)
+        U238Pb206 <- 1/X[i,'Pb206U238']
+        Pb207Pb206 <- X[i,'Pb207U235']/(X[i,'Pb206U238']*U238U235)
         J <- matrix(0,2,2)
-        J[1,2] <- -1/x$x[i,'Pb206U238']^2
-        J[2,1] <- 1/(x$x[i,'Pb206U238']*U238U235)
-        J[2,2] <- -Pb207Pb206/x$x[i,'Pb206U238']
-        E <- cor2cov2(x$x[i,'errPb207U235'],x$x[i,'errPb206U238'],x$x[i,'rhoXY'])
+        J[1,2] <- -1/X[i,'Pb206U238']^2
+        J[2,1] <- 1/(X[i,'Pb206U238']*U238U235)
+        J[2,2] <- -Pb207Pb206/X[i,'Pb206U238']
+        E <- cor2cov2(X[i,'errPb207U235'],X[i,'errPb206U238'],X[i,'rhoXY'])
         out$x <- c(U238Pb206,Pb207Pb206)
         out$cov <- J %*% E %*% t(J)
-    } else if (x$format == 2){
-        out$x <- x$x[i,labels]
+    } else if (format == 2){
+        out$x <- X[i,labels]
         out$cov <- matrix(0,2,2)
-        diag(out$cov) <- x$x[i,c('errU238Pb206','errPb207Pb206')]^2
-        out$cov <- cor2cov2(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],x$x[i,'rhoXY'])
-    } else if (x$format == 3){
-        U238Pb206 <- 1/x$x[i,'Pb206U238']
-        out$x <- c(U238Pb206,x$x[i,'Pb207Pb206'])
+        diag(out$cov) <- X[i,c('errU238Pb206','errPb207Pb206')]^2
+        out$cov <- cor2cov2(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],X[i,'rhoXY'])
+    } else if (format == 3){
+        U238Pb206 <- 1/X[i,'Pb206U238']
+        out$x <- c(U238Pb206,X[i,'Pb207Pb206'])
         J <- matrix(0,2,2)
-        E <- cor2cov2(x$x[i,'errPb206U238'],x$x[i,'errPb207Pb206'],x$x[i,'rhoYZ'])
+        E <- cor2cov2(X[i,'errPb206U238'],X[i,'errPb207Pb206'],X[i,'rhoYZ'])
         J[1,1] <- -U238Pb206^2
         J[2,2] <- 1
         out$cov <- J %*% E %*% t(J)
-    } else if (x$format == 4){
+    } else if (format == 4){
         U238U235 <- iratio('U238U235')[1]
-        U238Pb206 <- 1/x$x[i,'Pb206U238']
-        Pb207Pb206 <- x$x[i,'Pb207U235']/(x$x[i,'Pb206U238']*U238U235)
-        Pb204Pb206 <- x$x[i,'Pb204U238']/x$x[i,'Pb206U238']
-        E <- cor2cov3(x$x[i,'errPb207U235'],x$x[i,'errPb206U238'],
-                      x$x[i,'errPb204U238'],x$x[i,'rhoXY'],
-                      x$x[i,'rhoXZ'],x$x[i,'rhoYZ'])
+        U238Pb206 <- 1/X[i,'Pb206U238']
+        Pb207Pb206 <- X[i,'Pb207U235']/(X[i,'Pb206U238']*U238U235)
+        Pb204Pb206 <- X[i,'Pb204U238']/X[i,'Pb206U238']
+        E <- cor2cov3(X[i,'errPb207U235'],X[i,'errPb206U238'],
+                      X[i,'errPb204U238'],X[i,'rhoXY'],
+                      X[i,'rhoXZ'],X[i,'rhoYZ'])
         J <- matrix(0,3,3)
-        J[1,2] <- -U238Pb206/x$x[i,'Pb206U238']
-        J[2,1] <- 1/(x$x[i,'Pb206U238']*U238U235)
-        J[2,2] <- -Pb207Pb206/x$x[i,'Pb206U238']
-        J[3,2] <- -Pb204Pb206/x$x[i,'Pb206U238']
-        J[3,3] <- 1/x$x[i,'Pb206U238']
+        J[1,2] <- -U238Pb206/X[i,'Pb206U238']
+        J[2,1] <- 1/(X[i,'Pb206U238']*U238U235)
+        J[2,2] <- -Pb207Pb206/X[i,'Pb206U238']
+        J[3,2] <- -Pb204Pb206/X[i,'Pb206U238']
+        J[3,3] <- 1/X[i,'Pb206U238']
         out$x <- c(U238Pb206,Pb207Pb206,Pb204Pb206)
         out$cov <- J %*% E %*% t(J)
-    } else if (x$format == 5){
-        out$x <- x$x[i,labels]
+    } else if (format == 5){
+        out$x <- X[i,labels]
         out$cov <- matrix(0,3,3)
-        diag(out$cov) <- x$x[i,c('errU238Pb206','errPb207Pb206','errPb204Pb206')]^2
+        diag(out$cov) <- X[i,c('errU238Pb206','errPb207Pb206','errPb204Pb206')]^2
         out$cov <-
-            cor2cov3(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],x$x[i,'errPb204Pb206'],
-                     x$x[i,'rhoXY'],x$x[i,'rhoXZ'],x$x[i,'rhoYZ'])
-    } else if (x$format == 6){
-        U238Pb206 <- 1/x$x[i,'Pb206U238']
-        Pb207Pb206 <- x$x[i,'Pb207Pb206']
-        Pb204Pb206 <- x$x[i,'Pb204Pb206']
+            cor2cov3(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],X[i,'errPb204Pb206'],
+                     X[i,'rhoXY'],X[i,'rhoXZ'],X[i,'rhoYZ'])
+    } else if (format == 6){
+        U238Pb206 <- 1/X[i,'Pb206U238']
+        Pb207Pb206 <- X[i,'Pb207Pb206']
+        Pb204Pb206 <- X[i,'Pb204Pb206']
         out$x <- c(U238Pb206,Pb207Pb206,Pb204Pb206)
         out$cov <- matrix(0,3,3)
-        out$cov[1,1] <- (U238Pb206*x$x[i,'errPb206U238']/x$x[i,'Pb206U238'])^2
-        out$cov[2,2] <- x$x[i,'errPb207Pb206']^2
-        out$cov[3,3] <- x$x[i,'errPb204Pb206']^2
-        out$cov[1,2] <- get.cov.76.86(x$x[i,'Pb207Pb206'],x$x[i,'errPb207Pb206'],
-                                      x$x[i,'Pb206U238'],x$x[i,'errPb206U238'],
-                                      x$x[i,'Pb207U235'],x$x[i,'errPb207U235'])
-        out$cov[1,3] <- get.cov.46.86(x$x[i,'Pb204Pb206'],x$x[i,'errPb204Pb206'],
-                                      x$x[i,'Pb206U238'],x$x[i,'errPb206U238'],
-                                      x$x[i,'Pb204U238'],x$x[i,'errPb204U238'])
-        out$cov[2,3] <- get.cov.46.76(x$x[i,'Pb204Pb206'],x$x[i,'errPb204Pb206'],
-                                      x$x[i,'Pb207Pb206'],x$x[i,'errPb207Pb206'],
-                                      x$x[i,'Pb204Pb207'],x$x[i,'errPb204Pb207'])
+        out$cov[1,1] <- (U238Pb206*X[i,'errPb206U238']/X[i,'Pb206U238'])^2
+        out$cov[2,2] <- X[i,'errPb207Pb206']^2
+        out$cov[3,3] <- X[i,'errPb204Pb206']^2
+        out$cov[1,2] <- get.cov.76.86(X[i,'Pb207Pb206'],X[i,'errPb207Pb206'],
+                                      X[i,'Pb206U238'],X[i,'errPb206U238'],
+                                      X[i,'Pb207U235'],X[i,'errPb207U235'])
+        out$cov[1,3] <- get.cov.46.86(X[i,'Pb204Pb206'],X[i,'errPb204Pb206'],
+                                      X[i,'Pb206U238'],X[i,'errPb206U238'],
+                                      X[i,'Pb204U238'],X[i,'errPb204U238'])
+        out$cov[2,3] <- get.cov.46.76(X[i,'Pb204Pb206'],X[i,'errPb204Pb206'],
+                                      X[i,'Pb207Pb206'],X[i,'errPb207Pb206'],
+                                      X[i,'Pb204Pb207'],X[i,'errPb204Pb207'])
         out$cov[2,1] <- out$cov[1,2]
         out$cov[3,1] <- out$cov[1,3]
         out$cov[3,2] <- out$cov[2,3]
-    } else if (x$format == 7){
-        out$x <- x$x[i,labels]
-        out$cov <- cor2cov4(x$x[i,'errU238Pb206'],x$x[i,'errPb207Pb206'],
-                            x$x[i,'errPb208Pb206'],x$x[i,'errTh232U238'],
-                            x$x[i,'rhoXY'],x$x[i,'rhoXZ'],x$x[i,'rhoXW'],
-                            x$x[i,'rhoYZ'],x$x[i,'rhoYW'],x$x[i,'rhoZW'])
+    } else if (format == 7){
+        U238U235 <- iratio('U238U235')[1]
+        U238Pb206 <- 1/X[i,'Pb206U238']
+        Pb207Pb206 <- X[i,'Pb207U235']/(X[i,'Pb206U238']*U238U235)
+        Pb208Pb206 <- X[i,'Pb208Th232']*X[i,'Th232U238']/X[i,'Pb206U238']
+        Th232U238 <- X[i,'Th232U238']
+        E <- cor2cov4(X[i,'errPb207U235'],X[i,'errPb206U238'],
+                      X[i,'errPb208Th232'],X[i,'errTh232U238'],
+                      X[i,'rhoXY'],X[i,'rhoXZ'],X[i,'rhoXW'],
+                      X[i,'rhoYZ'],X[i,'rhoYW'],X[i,'rhoZW'])
+        J <- matrix(0,4,4)
+        J[1,2] <- -U238Pb206/X[i,'Pb206U238']
+        J[2,1] <- 1/(X[i,'Pb206U238']*U238U235)
+        J[2,2] <- -Pb207Pb206/X[i,'Pb206U238']
+        J[3,2] <- -Pb208Pb206/X[i,'Pb206U238']
+        J[3,3] <- X[i,'Th232U238']/X[i,'Pb206U238']
+        J[3,4] <- X[i,'Pb208Th232']/X[i,'Pb206U238']
+        J[4,4] <- 1
+        out$x <- c(U238Pb206,Pb207Pb206,Pb208Pb206,Th232U238)
+        out$cov <- J %*% E %*% t(J)
+    } else if (format == 8){
+        out$x <- X[i,labels]
+        out$cov <- cor2cov4(X[i,'errU238Pb206'],X[i,'errPb207Pb206'],
+                            X[i,'errPb208Pb206'],X[i,'errTh232U238'],
+                            X[i,'rhoXY'],X[i,'rhoXZ'],X[i,'rhoXW'],
+                            X[i,'rhoYZ'],X[i,'rhoYW'],X[i,'rhoZW'])
     }
     names(out$x) <- labels
     colnames(out$cov) <- labels
@@ -175,7 +213,7 @@ get.UPb.isochron.ratios.204 <- function(x,i=NA){
     if (x$format%in%c(4,5,6)){
         labels <- c('U238Pb206','Pb204Pb206','U235Pb207','Pb204Pb207')
     } else {
-        stop('Incorrect input format for the get.UPb.isochron.ratios function.')
+        stop('Format does not contain 204Pb function.')
     }
     if (all(is.na(i))){
         ns <- length(x)
@@ -208,7 +246,7 @@ get.UPb.isochron.ratios.204 <- function(x,i=NA){
     out
 }
 get.UPb.isochron.ratios.208 <- function(x,i=NA,tt=0){
-    if (x$format==7){
+    if (x$format>6){
         labels <- c('U238Pb206','Pb208cPb206','U235Pb207',
                     'Pb208cPb207','Th232U238','Th232Pb208',
                     'Pb206cPb208','Pb207cPb208')
@@ -269,92 +307,87 @@ get.UPb.isochron.ratios.208 <- function(x,i=NA,tt=0){
     out
 }
 
-w2tw <- function(w){
-    U <- iratio('U238U235')[1]
-    Pb207U235 <- w[,1]
-    errPb207U235 <- w[,2]
-    Pb206U238 <- w[,3]
-    errPb206U238 <- w[,4]
-    rho <- w[,5]
-    U238Pb206 <- 1/Pb206U238
-    Pb207Pb206 <- Pb207U235/(U*Pb206U238)
-    J11 <- 0*U238Pb206
-    J12 <- -U238Pb206/Pb206U238
-    J21 <- 1/(U*Pb206U238)
-    J22 <- -Pb207Pb206/Pb206U238
-    E11 <- errPb207U235^2
-    E22 <- errPb206U238^2
-    E12 <- rho*errPb207U235*errPb206U238
-    err <- errorprop(J11,J12,J21,J22,E11,E22,E12)
-    errU238Pb206 <- sqrt(err[,'varX'])
-    errPb207Pb206 <- sqrt(err[,'varY'])
-    rho <- err[,'cov']/(errU238Pb206*errPb207Pb206)
-    out <- cbind(U238Pb206,errU238Pb206,Pb207Pb206,errPb207Pb206,rho)
-    colnames(out) <- c('U238Pb206','errU238Pb206',
-                       'Pb207Pb206','errPb207Pb206','rhoXY')
-    out
-}
-tw2w <- function(tw){
-    U <- iratio('U238U235')[1]
-    U238Pb206 <- tw[,1]
-    errU238Pb206 <- tw[,2]
-    Pb207Pb206 <- tw[,3]
-    errPb207Pb206 <- tw[,4]
-    rho <- tw[,5]
-    Pb207U235 <- U*Pb207Pb206/U238Pb206
-    Pb206U238 <- 1/U238Pb206
-    J11 <- -Pb207U235/U238Pb206
-    J12 <- U/U238Pb206
-    J21 <- -1/U238Pb206^2
-    J22 <- 0*Pb206U238
-    E11 <- errU238Pb206^2
-    E22 <- errPb207Pb206^2
-    E12 <- rho*errU238Pb206*errPb207Pb206
-    err <- errorprop(J11,J12,J21,J22,E11,E22,E12)
-    errPb207U235 <- sqrt(err[,'varX'])
-    errPb206U238 <- sqrt(err[,'varY'])
-    rho <- err[,'cov']/(errPb207U235*errPb206U238)
-    out <- cbind(Pb207U235,errPb207U235,Pb206U238,errPb206U238,rho)
-    colnames(out) <- c('Pb207U235','errPb207U235',
-                       'Pb206U238','errPb206U238','rhoXY')
+w2tw <- function(w,format){
+    if (format %in% c(1,2,3)){
+        cnames <- c('U238Pb206','errU238Pb206',
+                    'Pb207Pb206','errPb207Pb206','rhoXY')
+    } else if (format%in%c(4,5,6)){
+        cnames <- c('U238Pb206','errU238Pb206',
+                    'Pb207Pb206','errPb207Pb206',
+                    'Pb206Pb206','errPb204Pb206',
+                    'rhoXY','rhoXZ','rhoYZ')
+    } else if (format%in%c(7,8)){
+        cnames <- c('U238Pb206','errU238Pb206',
+                    'Pb207Pb206','errPb207Pb206',
+                    'Pb208Pb206','errPb208Pb206',
+                    'Th232U238','errTh232U238',
+                    'rhoXY','rhoXZ','rhoXW',
+                    'rhoYZ','rhoYW','rhoZW')
+    } else {
+        stop('Invalid input format.')
+    }
+    ns <- nrow(w)
+    out <- w*0
+    colnames(out) <- cnames
+    for (i in 1:ns){
+        tw <- tera.wasserburg(x=w,i=i,format=format)
+        out[i,] <- wtw_helper(x=tw$x,covmat=tw$cov,cnames=cnames)
+    }
     out
 }
 
-# convert data to a 5-column table for concordia analysis
-flat.UPb.table <- function(x,type=1){
-    ns <- length(x)
-    out <- matrix(0,ns,5)
-    if (type==1){
-        colnames(out) <- c('Pb207U235','errPb207U235',
-                           'Pb206U238','errPb206U238','rhoXY')
-    } else if (type==2){
-        colnames(out) <- c('U238Pb206','errU238Pb206',
-                           'Pb207Pb206','errPb207Pb206','rhoXY')
-    } else if (type==3){
-        colnames(out) <- c('Pb206U238','errPb206U238',
-                           'Pb208Th232','errPb208Th232','rhoXY')
+tw2w <- function(tw,format){
+    if (format %in% c(1,2,3)){
+        cnames <- c('Pb207U235','errPb207U235',
+                    'Pb206U238','errPb206U238','rhoXY')
+    } else if (format%in%c(4,5,6)){
+        cnames <- c('Pb207U235','errPb207U235',
+                    'Pb206U238','errPb206U238',
+                    'Pb204U238','errPb204U238',
+                    'rhoXY','rhoXZ','rhoYZ')
+    } else if (format%in%c(7,8)){
+        cnames <- c('Pb207U235','errPb207U235',
+                    'Pb206U238','errPb206U238',
+                    'Pb208Th232','errPb208Th232',
+                    'Th232U238','errTh232U238',
+                    'rhoXY','rhoXZ','rhoXW',
+                    'rhoYZ','rhoYW','rhoZW')
     } else {
-        stop('Incorrect concordia type.')
+        stop('Invalid input format.')
     }
+    ns <- nrow(tw)
+    out <- tw*0
+    colnames(out) <- cnames
     for (i in 1:ns){
-        if (type==1){
-            xi <- wetherill(x,i)
-            j1 <- 'Pb207U235'
-            j2 <- 'Pb206U238'
-        } else if (type==2){
-            xi <- tera.wasserburg(x,i)
-            j1 <- 'U238Pb206'
-            j2 <- 'Pb207Pb206'
-        } else if (type==3){
-            xi <- wetherill(x,i)
-            j1 <- 'Pb206U238'
-            j2 <- 'Pb208Th232'
-        }
-        out[i,1] <- xi$x[j1]
-        out[i,3] <- xi$x[j2]
-        out[i,2] <- sqrt(xi$cov[j1,j1])
-        out[i,4] <- sqrt(xi$cov[j2,j2])
-        out[i,5] <- xi$cov[j1,j2]/(out[i,2]*out[i,4])
+        w <- wetherill(x=tw,i=i,format=format)
+        out[i,] <- wtw_helper(x=w$x,covmat=w$cov,cnames=cnames)
+    }
+    out
+}
+
+wtw_helper <- function(x,covmat,cnames){
+    nc <- length(cnames)
+    out <- rep(0,nc)
+    names(out) <- cnames
+    err <- sqrt(diag(covmat))
+    cormat <- 0*covmat
+    pos <- which(diag(covmat)>0)
+    cormat[pos,pos] <- stats::cov2cor(covmat[pos,pos])
+    out[c(1,3)] <- x[1:2]
+    out[c(2,4)] <- err[1:2]
+    out['rhoXY'] <- cormat[1,2]
+    if (nc>5){
+        out[5] <- x[3]
+        out[6] <- err[3]
+        out['rhoXZ'] <- cormat[1,3]
+        out['rhoYZ'] <- cormat[2,3]
+    }
+    if (nc>9){
+        out[7] <- x[4]
+        out[8] <- err[4]
+        out['rhoXW'] <- cormat[1,4]
+        out['rhoYW'] <- cormat[2,4]
+        out['rhoZW'] <- cormat[3,4]
     }
     out
 }
@@ -577,16 +610,16 @@ check.zero.UPb <- function(tt){
 
 get.Pb204U238.ratios <- function(x){
     labels <- c('Pb204U238','errPb204U238')
-    if (x$format %in% c(1,2,3,7)){
-        stop('No 204Pb measurements available!')
-    } else if (x$format %in% c(4,6)){
+    if (x$format %in% c(4,6)){
         out <- subset(x$x,select=labels)
-    } else {
+    } else if (x$format == 5){
         Pb204U238 <- x$x[,'Pb204Pb206']/x$x[,'U238Pb206']
         errPb204U238 <-
             Pb204U238*sqrt( (x$x[,'errPb204Pb206']/x$x[,'Pb204Pb206'])^2 +
                             (x$x[,'errU238Pb206']/x$x[,'U238Pb206'])^2 )
         out <- cbind(Pb204U238,errPb204U238)
+    } else {
+        stop('No 204Pb measurements available!')
     }
     colnames(out) <- labels
     out
@@ -596,9 +629,9 @@ get.Pb207U235.ratios <- function(x,exterr=FALSE){
     out <- matrix(0,ns,2)
     labels <- c('Pb207U235','errPb207U235')
     colnames(out) <- labels
-    if (x$format %in% c(1,3,4,6)){
+    if (x$format %in% c(1,3,4,6,7)){
         out <- subset(x$x,select=labels)
-    } else if (x$format %in% c(2,5,7)){
+    } else if (x$format %in% c(2,5,8)){
         R <- iratio('U238U235')[1]
         sR <- iratio('U238U235')[2]
         X <- x$x[,'U238Pb206']
@@ -618,11 +651,11 @@ get.U235Pb207.ratios <- function(x,exterr=FALSE){
     out <- matrix(0,ns,2)
     labels <- c('U235Pb207','errU235Pb207')
     colnames(out) <- labels
-    if (x$format %in% c(1,3,4,6)){
+    if (x$format %in% c(1,3,4,6,7)){
         out[,'U235Pb207'] <- 1/x$x[,'Pb207U235']
         out[,'errU235Pb207'] <- out[,'U235Pb207']*
             x$x[,'errPb207U235']/x$x[,'Pb207U235']
-    } else if (x$format %in% c(2,5,7)){
+    } else if (x$format %in% c(2,5,8)){
         R <- iratio('U238U235')[1]
         sR <- iratio('U238U235')[2]
         X <- x$x[,'U238Pb206']
@@ -642,9 +675,9 @@ get.Pb206U238.ratios <- function(x){
     out <- matrix(0,ns,2)
     labels <- c('Pb206U238','errPb206U238')
     colnames(out) <- labels
-    if (x$format %in% c(1,3,4,6)){
+    if (x$format %in% c(1,3,4,6,7)){
         out <- subset(x$x,select=labels)
-    } else if (x$format %in% c(2,5,7)){
+    } else if (x$format %in% c(2,5,8)){
         out[,'Pb206U238'] <- 1/x$x[,'U238Pb206']
         out[,'errPb206U238'] <- out[,'Pb206U238']*
             x$x[,'errU238Pb206']/x$x[,'U238Pb206']
@@ -656,11 +689,11 @@ get.U238Pb206.ratios <- function(x){
     out <- matrix(0,ns,2)
     labels <- c('U238Pb206','errU238Pb206')
     colnames(out) <- labels
-    if (x$format %in% c(1,3,4,6)){
+    if (x$format %in% c(1,3,4,6,7)){
         out[,'U238Pb206'] <- 1/x$x[,'Pb206U238']
         out[,'errU238Pb206'] <- out[,'U238Pb206']*
             x$x[,'errPb206U238']/x$x[,'Pb206U238']
-    } else if (x$format %in% c(2,5,7)){
+    } else if (x$format %in% c(2,5,8)){
         out <- subset(x$x,select=labels)
     }
     out
@@ -670,7 +703,7 @@ get.Pb207Pb206.ratios <- function(x,exterr=FALSE){
     out <- matrix(0,ns,2)
     labels <- c('Pb207Pb206','errPb207Pb206')
     colnames(out) <- labels
-    if (x$format %in% c(1,4)){
+    if (x$format %in% c(1,4,7)){
         R <- iratio('U238U235')[1]
         sR <- iratio('U238U235')[2]
         X <- x$x[,'Pb207U235']
@@ -683,17 +716,19 @@ get.Pb207Pb206.ratios <- function(x,exterr=FALSE){
         relerr2 <- (sX/X)^2 - 2*covXY/(X*Y) + (sY/Y)^2
         if (exterr) relerr2 <- relerr2 + (sR/R)^2
         out[,'errPb207Pb206'] <- sqrt(relerr2)*out[,'Pb207Pb206']
-    } else if (x$format %in% c(2,3,5,6,7)){
+    } else if (x$format %in% c(2,3,5,6,8)){
         out <- subset(x$x,select=labels)
     }
     out
 }
 get.Pb208Th232.ratios <- function(x){
-    ns <- length(x)
-    out <- matrix(0,ns,2)
     labels <- c('Pb208Th232','errPb208Th232')
-    colnames(out) <- labels
     if (x$format == 7){
+        out <- x$x[,labels]
+    } else if (x$format == 8){
+        ns <- length(x)
+        out <- matrix(0,ns,2)
+        colnames(out) <- labels
         out[,'Pb208Th232'] <- x$x[,'Pb208Pb206']/(x$x[,'U238Pb206']*x$x[,'Th232U238'])
         J1 <- -out[,'Pb208Th232']/x$x[,'U238Pb206']
         J2 <- 1/(x$x[,'U238Pb206']*x$x[,'Th232U238'])
@@ -711,8 +746,24 @@ get.Pb208Th232.ratios <- function(x){
     out
 }
 get.Pb208Pb206.ratios <- function(x){
+    labels <- c('Pb208Pb206','errPb208Pb206')
     if (x$format == 7){
-        out <- x$x[,c('Pb208Pb206','errPb208Pb206')]
+        ns <- length(x)
+        out <- matrix(0,ns,2)
+        colnames(out) <- labels
+        out[,'Pb208Pb206'] <- x$x[,'Pb208Th232']*x$x[,'Th232U238']/x$x[,'Pb206U238']
+        J1 <- -out[,'Pb208Pb206']/x$x[,'Pb206U238']
+        J2 <- x$x[,'Th232U238']/x$x[,'Pb206U238']
+        J3 <- x$x[,'Pb208Th232']/x$x[,'Pb206U238']
+        E11 <- x$x[,'errPb206U238']^2
+        E22 <- x$x[,'errPb208Th232']^2
+        E33 <- x$x[,'Th232U238']^2
+        E12 <- x$x[,'rhoXZ']*x$x[,'Pb206U238']*x$x[,'Pb208Th232']
+        E13 <- x$x[,'rhoXW']*x$x[,'Pb206U238']*x$x[,'Th232U238']
+        E23 <- x$x[,'rhoZW']*x$x[,'Pb208Th232']*x$x[,'Th232U238']
+        out[,'errPb208Th232'] <- errorprop1x3(J1,J2,J3,E11,E22,E33,E12,E13,E23)
+    } else if (x$format == 8){
+        out <- x$x[,labels]
     } else {
         stop('Wrong input format: no Pb208 present in this dataset.')
     }
@@ -754,7 +805,7 @@ get.Pb207U235.age.default <- function(x,sx=0,exterr=FALSE,d=diseq(),...){
     }
     out
 }
-get.Pb207U235.age.UPb <- function(x,i,exterr=FALSE,...){
+get.Pb207U235.age.UPb <- function(x,i=1,exterr=FALSE,...){
     r75 <- get.Pb207U235.ratios(x)
     get.Pb207U235.age(r75[i,'Pb207U235'],r75[i,'errPb207U235'],exterr=exterr,d=x$d)
 }
@@ -877,7 +928,7 @@ get.Pb207Pb206.age.default <- function(x,sx=0,exterr=FALSE,d=diseq(),t.68=NA,...
     }
     out    
 }
-get.Pb207Pb206.age.UPb <- function(x,i,exterr=FALSE,...){
+get.Pb207Pb206.age.UPb <- function(x,i=1,exterr=FALSE,...){
     r76 <- get.Pb207Pb206.ratios(x)
     get.Pb207Pb206.age(r76[i,'Pb207Pb206'],r76[i,'errPb207Pb206'],
                        exterr=exterr,d=x$d[i],...)
@@ -944,103 +995,76 @@ get.Pb208Th232.age.UPb <- function(x,i=NA,exterr=FALSE,...){
 # x is an object of class \code{UPb}
 # returns a matrix of 7/5, 6/8, 7/6
 # and concordia ages and their uncertainties.
-UPb.age <- function(x,exterr=FALSE,i=NA,sigdig=NA,conc=TRUE,
-                    show.p=FALSE,common.Pb=0,...){
-    if (common.Pb>0) X <- Pb0corr(x,option=common.Pb)
-    else X <- x
+UPb.age <- function(x,exterr=FALSE,i=NA,sigdig=NA,conc=TRUE,omit4c=NULL,
+                    discordance=discfilter(),common.Pb=0,...){
+    if (discordance$option==0 | discordance$before) xd <- x
+    else xd <- Pb0corr(x,option=common.Pb,omit4c=omit4c)
+    if (common.Pb==0) X <- x
+    else X <- Pb0corr(x,option=common.Pb,omit4c=omit4c)
+    if (!is.na(i)){
+        out <- UPb_age_helper(x=x,X=X,xd=xd,i=i,exterr=exterr,sigdig=sigdig,
+                              conc=conc,discordance=discordance)
+    } else {
+        nn <- length(x)
+        out <- NULL
+        for (i in 1:nn){
+            ti <- UPb_age_helper(x=x,X=X,xd=xd,i=i,exterr=exterr,sigdig=sigdig,
+                                 conc=conc,discordance=discordance)
+            out <- rbind(out,ti)
+        }
+    }
+    out
+}
+
+# x = raw data
+# X = common Pb corrected data (if common.Pb>0)
+# xd = discordant data (raw if before==FALSE, common Pb corrected if before==FALSE)
+UPb_age_helper <- function(x,X,xd,i=1,exterr=FALSE,sigdig=NA,conc=TRUE,
+                           discordance=discfilter(),...){
+    Xi <- subset(X,subset=((1:length(X))%in%i))
     labels <- c('t.75','s[t.75]','t.68','s[t.68]','t.76','s[t.76]')
-    hasTh <- x$format==7
+    hasTh <- (x$format>6)
     if (hasTh) labels <- c(labels,'t.82','s[t.82]')
     if (conc) labels <- c(labels,'t.conc','s[t.conc]')
-    if (conc & show.p) labels <- c(labels,'p[conc]')
-    if (!is.na(i)){
-        t.75 <- get.Pb207U235.age(X,i,exterr=exterr)
-        t.68 <- get.Pb206U238.age(X,i,exterr=exterr)
-        t.76 <- get.Pb207Pb206.age(X,i,exterr=exterr,t.68=t.68[1])
-        t.75.out <- roundit(t.75[1],t.75[2],sigdig=sigdig)
-        t.68.out <- roundit(t.68[1],t.68[2],sigdig=sigdig)
-        t.76.out <- roundit(t.76[1],t.76[2],sigdig=sigdig)
-        out <- c(t.75.out,t.68.out,t.76.out)
-        if (hasTh){
-            t.82 <- get.Pb208Th232.age(X,i,exterr=exterr)
-            t.82.out <- roundit(t.82[1],t.82[2],sigdig=sigdig)
-            out <- c(out,t.82.out)
-        }
-        if (conc){
-            t.conc <- concordia.age(x=X,i=i,exterr=exterr)
-            t.conc.out <- roundit(t.conc$age[1],t.conc$age[2],sigdig=sigdig)
-            out <- c(out,t.conc.out)
-        }
-        if (conc & show.p){
-            SS.concordance <-
-                LL.concordia.age(tt=t.conc$age[1],cc=wetherill(X,i),
-                                 mswd=TRUE,exterr=exterr,d=x$d)
-            p.value <- 1-stats::pchisq(SS.concordance,1)
-            if (!is.na(sigdig)) p.value <- signif(p.value,sigdig)
-            out <- c(out,p.value)
-        }
-        names(out) <- labels
-    } else {
-        nn <- nrow(X$x)
-        out <- matrix(0,nn,length(labels))
-        for (i in 1:nn){
-            out[i,] <- UPb.age(X,i=i,exterr=exterr,sigdig=sigdig,
-                               conc=conc,show.p=show.p)
-        }
-        colnames(out) <- labels
+    tlabels <- labels
+    if (discordance$option%in%c(1,'t',2,'r',3,'sk',4,'a',5,'c'))
+        labels <- c(labels,'disc')
+    if (discordance$option%in%c(6,'p'))
+        labels <- c(labels,'p[conc]')
+    t.75 <- get.Pb207U235.age(Xi,exterr=exterr)
+    t.68 <- get.Pb206U238.age(Xi,exterr=exterr)
+    t.76 <- get.Pb207Pb206.age(Xi,exterr=exterr,t.68=subset(t.68,select=1))
+    t.75.out <- roundit(t.75[1],t.75[2],sigdig=sigdig)
+    t.68.out <- roundit(t.68[1],t.68[2],sigdig=sigdig)
+    t.76.out <- roundit(t.76[1],t.76[2],sigdig=sigdig)
+    out <- c(t.75.out,t.68.out,t.76.out)
+    if (hasTh){
+        t.82 <- get.Pb208Th232.age(Xi,exterr=exterr)
+        t.82.out <- roundit(t.82[1],t.82[2],sigdig=sigdig)
+        out <- c(out,t.82.out)
     }
+    if (conc){
+        t.conc <- concordia.age(x=Xi,i=1,exterr=exterr)
+        t.conc.out <- roundit(t.conc$age[1],t.conc$age[2],sigdig=sigdig)
+        out <- c(out,t.conc.out)
+    }
+    if (discordance$option>0){
+        xdi <- subset(xd,subset=((1:length(xd))%in%i))
+    }
+    if (discordance$option%in%c(1,'t',2,'r',3,'sk',4,'a',5,'c')){
+        xi <- subset(x,subset=((1:length(x))%in%i))
+        dif <- discordance(x=xi,X=xdi,option=discordance$option)
+        out <- c(out,roundit(dif,sigdig=sigdig))
+    }
+    if (discordance$option%in%c(6,'p')){
+        t.conc <- concordia.age(x=xdi,exterr=exterr)
+        SS.concordance <-
+            LL.concordia.age(tt=t.conc$age[1],cc=wetherill(xdi,i=1),
+                             mswd=TRUE,exterr=exterr,d=xdi$d)
+        p.value <- 1-stats::pchisq(SS.concordance,1)
+        p.value <- roundit(p.value,sigdig=sigdig)
+        out <- c(out,p.value)
+    }
+    names(out) <- labels
     out
 }
-
-filter.UPb.ages <- function(x,type=4,cutoff.76=1100,exterr=FALSE,
-                            cutoff.disc=list(-15,5,TRUE),common.Pb=0){
-    tout <- UPb.age(x,exterr=exterr,conc=(type==5),common.Pb=common.Pb)
-    if (any(is.na(cutoff.disc))){
-        is.concordant <- rep(TRUE,length(x))
-    } else {
-        # apply cutoff filter before common Pb correction?
-        if (cutoff.disc[[3]] & common.Pb>0){
-            tin <- UPb.age(x,exterr=exterr,conc=(type==5),common.Pb=0)
-        } else { # apply cutoff filter after common Pb correction.
-            tin <- tout
-        }
-        is.concordant <- concordant(tin,cutoff.76=cutoff.76,cutoff.disc=cutoff.disc)
-        if (!any(is.concordant)){
-            stop(paste0('There are no concordant grains in this sample.',
-                        'Try adjusting the discordance limits OR ',
-                        'apply a common-Pb correction OR ',
-                        '(if you have already applied a common-Pb correction), ',
-                        'apply the discordance filter before the ',
-                        'common-Pb correction.'))
-        }
-    }
-    out <- matrix(NA,length(x),2)
-    if (type==1){
-        out[is.concordant,] <- tout[is.concordant,c('t.75','s[t.75]'),drop=FALSE]
-    } else if (type==2){
-        out[is.concordant,] <- tout[is.concordant,c('t.68','s[t.68]'),drop=FALSE]
-    } else if (type==3){
-        out[is.concordant,] <- tout[is.concordant,c('t.76','s[t.76]'),drop=FALSE]
-    } else if (type==4){
-        do.76 <- (tout[,'t.68']>cutoff.76)
-        i.76 <- as.vector(which(do.76 & is.concordant))
-        i.68 <- as.vector(which(!do.76 & is.concordant))
-        out[i.76,] <- tout[i.76,c('t.76','s[t.76]'),drop=FALSE]
-        out[i.68,] <- tout[i.68,c('t.68','s[t.68]'),drop=FALSE]
-    } else if (type==5){
-        out[is.concordant,] <- tout[is.concordant,c('t.conc','s[t.conc]'),drop=FALSE]
-    } else if (type==6){
-        out[is.concordant,] <- tout[is.concordant,c('t.82','s[t.82]'),drop=FALSE]
-    }
-    colnames(out) <- c('t','s[t]')
-    out
-}
-
-concordant <- function(tt,cutoff.76=1100,cutoff.disc=list(-15,5,TRUE)){
-    do.76 <- (tt[,'t.68'] > cutoff.76)
-    disc.75.68 <- 100*(1-tt[,'t.75']/tt[,'t.68'])
-    disc.68.76 <- 100*(1-tt[,'t.68']/tt[,'t.76'])
-    conc.75.68 <- !do.76 & (disc.75.68>cutoff.disc[[1]]) & (disc.75.68<cutoff.disc[[2]])
-    conc.68.76 <- do.76 & (disc.68.76>cutoff.disc[[1]]) & (disc.68.76<cutoff.disc[[2]])
-    conc.75.68 | conc.68.76
-}
