@@ -23,8 +23,8 @@
 #' \eqn{^{207}}Pb/\eqn{^{235}}U-ratios against each other (`Wetherill'
 #' diagram); or, equivalently, the \eqn{^{207}}Pb/\eqn{^{206}}Pb- and
 #' \eqn{^{206}}Pb/\eqn{^{238}}U-ratios (`Tera-Wasserburg'
-#' diagram). Alternatively, for data format 7, it is also possible to
-#' plot \eqn{^{208}}Pb/\eqn{^{232}}Th against the
+#' diagram). Alternatively, for data format 7 and 8, it is also
+#' possible to plot \eqn{^{208}}Pb/\eqn{^{232}}Th against the
 #' \eqn{^{206}}Pb/\eqn{^{238}}U.  The space of concordant isotopic
 #' compositions is marked by a curve, the `concordia line'. Isotopic
 #' ratio measurements are shown as 100(1-\code{alpha})\% confidence
@@ -52,7 +52,7 @@
 #'
 #' \code{3}: U-Th-Pb concordia -- \eqn{{}^{208}}Pb/\eqn{{}^{232}}Th
 #' vs. \eqn{{}^{206}}Pb/\eqn{{}^{238}}U (only available if
-#' \code{x$format=7})
+#' \code{x$format=7} or \code{8})
 #' 
 #' @param show.numbers logical flag (\code{TRUE} to show grain
 #'     numbers)
@@ -63,7 +63,7 @@
 #' @param clabel label for the colour legend (only used if
 #'     \code{levels} is not \code{NA}).
 #' 
-#' @param ellipse.col
+#' @param ellipse.fill
 #' Fill colour for the error ellipses. This can either be a single
 #' colour or multiple colours to form a colour ramp. Examples:
 #'
@@ -80,7 +80,11 @@
 #' a reversed palette: \code{rev(topo.colors(n=100,alpha=0.5))},
 #' etc.
 #'
-#' For empty ellipses, set \code{ellipse.col=NA}
+#' For empty ellipses, set \code{ellipse.fill=NA}
+#' 
+#' @param ellipse.stroke the stroke colour for the error
+#'     ellipses. Follows the same formatting guidelines as
+#'     \code{ellipse.fill}
 #' 
 #' @param concordia.col colour of the concordia line
 #' 
@@ -123,7 +127,7 @@
 #' \code{settings('iratio','Pb207Pb204')} (if \code{3<x$format<7}); or
 #' 
 #' \code{settings('iratio','Pb208Pb206')} and
-#' \code{settings('iratio','Pb208Pb207')} (if \code{x$format=7}).
+#' \code{settings('iratio','Pb208Pb207')} (if \code{x$format>6}).
 #' 
 #' \code{2}: use the isochron intercept as the initial Pb-composition
 #'
@@ -154,8 +158,10 @@
 #'     from the concordia diagram
 #' @param omit vector with indices of aliquots that should be plotted
 #'     but omitted from concordia or discordia age calculation
-#' @param omit.col colour that should be used for the omitted
+#' @param omit.fill fill colour that should be used for the omitted
 #'     aliquots.
+#' @param omit.stroke stroke colour that should be used for the
+#'     omitted aliquots.
 #' @param ... optional arguments to the generic \code{plot} function
 #'
 #' @return
@@ -257,11 +263,12 @@
 #' @export
 concordia <- function(x=NULL,tlim=NULL,alpha=0.05,type=1,
                       show.numbers=FALSE,levels=NA,clabel="",
-                      ellipse.col=c("#00FF0080","#FF000080"),
+                      ellipse.fill=c("#00FF0080","#FF000080"),
+                      ellipse.stroke='black',
                       concordia.col='darksalmon',exterr=FALSE,
                       show.age=0,sigdig=2,common.Pb=0,ticks=5,
                       anchor=list(FALSE,NA),hide=NULL,omit=NULL,
-                      omit.col=NA,...){
+                      omit.fill=NA,omit.stroke='grey',...){    
     if (is.null(x)){
         emptyconcordia(tlim=tlim,alpha=alpha,type=type,exterr=exterr,
                        concordia.col=concordia.col,ticks=ticks,...)
@@ -271,7 +278,7 @@ concordia <- function(x=NULL,tlim=NULL,alpha=0.05,type=1,
     plotit <- (1:ns)%ni%hide
     calcit <- (1:ns)%ni%c(hide,omit)
     if (common.Pb<1) X <- x
-    else X <- Pb0corr(x,option=common.Pb,omit=unique(c(hide,omit)))
+    else X <- Pb0corr(x,option=common.Pb,omit4c=unique(c(hide,omit)))
     X2plot <- subset(X,subset=plotit)
     lims <- prepare.concordia.line(x=X2plot,tlim=tlim,type=type,...)
     fit <- NULL
@@ -288,12 +295,14 @@ concordia <- function(x=NULL,tlim=NULL,alpha=0.05,type=1,
                         alpha=alpha,exterr=exterr,ticks=ticks)
     if (type==1) y <- data2york(X,option=1)
     else if (type==2) y <- data2york(X,option=2)
-    else if (x$format==7 & type==3) y <- data2york(X,option=5)
+    else if (x$format%in%c(7,8) & type==3) y <- data2york(X,option=5)
     else stop('Concordia type incompatible with this input format.')
     scatterplot(y,alpha=alpha,show.numbers=show.numbers,
                 show.ellipses=1*(show.age!=3),levels=levels,
-                clabel=clabel,ellipse.col=ellipse.col,add=TRUE,
-                hide=hide,omit=omit,omit.col=omit.col,addcolourbar=FALSE,...)
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,add=TRUE,
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,addcolourbar=FALSE,...)
     if (show.age==1){
         X2calc <- subset(X,subset=calcit)
         fit <- concordia.age(X2calc,type=type,exterr=exterr,alpha=alpha)
@@ -302,7 +311,10 @@ concordia <- function(x=NULL,tlim=NULL,alpha=0.05,type=1,
         fit$n <- length(X2calc)
         graphics::title(concordia.title(fit,sigdig=sigdig))
     }
-    colourbar(z=levels[calcit],col=ellipse.col,clabel=clabel)
+    # must be added to the end because otherwise R doesn't
+    # add the concordia ellipse to the scatterplot
+    colourbar(z=levels[calcit],fill=ellipse.fill,
+              stroke=ellipse.stroke,clabel=clabel)
     invisible(fit)
 }
 
@@ -532,7 +544,7 @@ concordia.title <- function(fit,sigdig=2,alpha=0.05,...){
         expr1 <- expression('concordia age ='~a%+-%b~'|'~c~'|'~d~'Ma (n='*n*')')
         list1$d <- rounded.age[4]
     }
-    line1 <- do.call('substitute',list(eval(expr1),list1))
+    line1 <- do.call(substitute,list(eval(expr1),list1))
     line2 <- substitute('MSWD ='~a~'|'~b~'|'~c~
                             ', p('*chi^2*') ='~d~'|'~e~'|'~f,
                         list(a=signif(fit$mswd['concordance'],2),
@@ -606,7 +618,21 @@ concordia_age_helper <- function(cc,d=diseq(),type=1,exterr=FALSE,...){
 
 # x has class 'UPb'
 concordia.comp <- function(x,type=1){
-    X <- flat.UPb.table(x,type=type)
+    if (type==1){
+        X <- data2york(x,option=1)
+        colnames(X) <- c('Pb207U235','errPb207U235',
+                         'Pb206U238','errPb206U238','rhoXY')
+    } else if (type==2){
+        X <- data2york(x,option=2)
+        colnames(X) <- c('U238Pb206','errU238Pb206',
+                         'Pb207Pb206','errPb207Pb206','rhoXY')
+    } else if (type==3){
+        X <- data2york(x,option=5)
+        colnames(X) <- c('Pb206U238','errPb206U238',
+                         'Pb208Th232','errPb208Th232','rhoXY')
+    } else {
+        stop('Incorrect concordia type.')
+    }
     out <- wtdmean2D(X)
     cnames <- colnames(X)[c(1,3)]
     names(out$x) <- cnames
@@ -751,17 +777,10 @@ emptyconcordia <- function(tlim=NULL,alpha=0.05,type=1,exterr=TRUE,
                          age_to_Pb207Pb206_ratio(tlim[2]),0,0))
         dat$format <- 2
     } else if (type==3){
-        Th232U238 <- 1
-        U238Pb206m <- age_to_U238Pb206_ratio(tlim[1])
-        U238Pb206M <- age_to_U238Pb206_ratio(tlim[2])
-        Pb207Pb206m <- age_to_Pb207Pb206_ratio(tlim[1])
-        Pb207Pb206M <- age_to_Pb207Pb206_ratio(tlim[2])
-        Pb208Th232m <- age_to_Pb208Th232_ratio(tlim[1])
-        Pb208Th232M <- age_to_Pb208Th232_ratio(tlim[2])
-        Pb208Pb206m <- Pb208Th232m*Th232U238*U238Pb206m
-        Pb208Pb206M <- Pb208Th232M*Th232U238*U238Pb206M
-        dat$x <- rbind(c(U238Pb206m,Pb207Pb206m,Pb208Pb206m,Th232U238,rep(0,6)),
-                       c(U238Pb206M,Pb207Pb206M,Pb208Pb206M,Th232U238,rep(0,6)))
+        dat$x <- rbind(c(0,0,age_to_Pb206U238_ratio(tlim[1]),0,
+                         age_to_Pb208Th232_ratio(tlim[1]),0,rep(0,8)),
+                       c(0,0,age_to_Pb206U238_ratio(tlim[2]),0,
+                         age_to_Pb208Th232_ratio(tlim[2]),0,rep(0,8)))
         dat$format <- 7
     } else {
         stop('Invalid concordia type.')
