@@ -91,10 +91,6 @@
 #' \code{UPb}, \code{ThPb}, \code{ReOs}, \code{RbSr}, \code{SmNd},
 #' \code{LuHf}, \code{UThHe} or \code{ThU}.
 #'
-#' @param xlim 2-element vector with the x-axis limits
-#'
-#' @param ylim 2-element vector with the y-axis limits
-#'
 #' @param alpha confidence cutoff for the error ellipses and
 #'     confidence intervals
 #'
@@ -108,7 +104,7 @@
 #'
 #' @param clabel label for the colour scale
 #'
-#' @param ellipse.col
+#' @param ellipse.fill
 #' Fill colour for the error ellipses. This can either be a single
 #' colour or multiple colours to form a colour ramp. Examples:
 #'
@@ -126,6 +122,10 @@
 #' etc.
 #'
 #' For empty ellipses, set \code{ellipse.col=NA}
+#' 
+#' @param ellipse.stroke the stroke colour for the error
+#'     ellipses. Follows the same formatting guidelines as
+#'     \code{ellipse.fill}
 #'
 #' @param ci.col the fill colour for the confidence interval of the
 #'     intercept and slope.
@@ -162,9 +162,12 @@
 #' @param omit vector with indices of aliquots that should be plotted
 #'     but omitted from the isochron age calculation.
 #' 
-#' @param omit.col colour that should be used for the omitted
+#' @param omit.fill fill colour that should be used for the omitted
 #'     aliquots.
 #' 
+#' @param omit.stroke stroke colour that should be used for the
+#'     omitted aliquots.
+#'
 #' @param ylab text label for the vertical plot axis
 #' 
 #' @param ... optional arguments to be passed on to the generic plot
@@ -272,8 +275,8 @@
 #'
 #' \item{p.value}{the p-value of a Chi-square test for linearity.}
 #'
-#' \item{fact}{the \eqn{100(1-\alpha/2)\%} confidence multiplier for
-#' the confidence intervals.}
+#' \item{fact}{the \eqn{100(1-\alpha/2)\%} multiplier for the confidence
+#' intervals.}
 #'
 #' \item{y0}{a four-element vector containing:
 #'
@@ -347,8 +350,8 @@
 #'
 #' \item{p.value}{the p-value of a Chi-square test for linearity.}
 #'
-#' \item{fact}{the \eqn{100(1-\alpha/2)\%} multiplier for the
-#' confidence intervals.}
+#' \item{fact}{the \eqn{100(1-\alpha/2)\%} multiplier for the confidence
+#' intervals.}
 #'
 #' \item{y0}{a three or four-element vector containing:
 #'
@@ -438,26 +441,26 @@
 isochron <- function(x,...){ UseMethod("isochron",x) }
 #' @rdname isochron
 #' @export
-isochron.default <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                             show.numbers=FALSE,levels=NA,clabel="",
-                             ellipse.col=c("#00FF0080","#FF000080"),
-                             ci.col='gray80',line.col='black',lwd=1,
-                             plot=TRUE,title=TRUE,model=1,
-                             show.ellipses=1*(model!=2),xlab='x',
-                             ylab='y',hide=NULL,omit=NULL,
-                             omit.col=NA,...){
+isochron.default <- function(x,alpha=0.05,sigdig=2,show.numbers=FALSE,
+                             levels=NA,clabel="",xlab='x',ylab='y',
+                             ellipse.fill=c("#00FF0080","#FF000080"),
+                             ellipse.stroke='black',ci.col='gray80',
+                             line.col='black',lwd=1,plot=TRUE,
+                             title=TRUE,model=1,show.ellipses=1*(model!=2),hide=NULL,
+                             omit=NULL,omit.fill=NA,omit.stroke='grey',...){
     d2calc <- clear(x,hide,omit)
     fit <- regression(data2york(d2calc),model=model)
     fit <- regression_init(fit,alpha=alpha)
     fit <- ci_isochron(fit)
     if (plot){
         y <- data2york(x)
-        scatterplot(y,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.ellipses=show.ellipses,
+        scatterplot(y,alpha=alpha,show.ellipses=show.ellipses,
                     show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=fit,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=fit,
                     ci.col=ci.col,line.col=line.col,lwd=lwd,
-                    hide=hide,omit=omit,omit.col=omit.col,...)
+                    hide=hide,omit=omit,omit.fill=omit.fill,
+                    omit.stroke=omit.stroke,...)
         if (title) graphics::title(isochrontitle(fit,sigdig=sigdig),
                                    xlab=xlab,ylab=ylab)
     }
@@ -482,14 +485,15 @@ isochron.default <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
 #'
 #' @rdname isochron
 #' @export
-isochron.UPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                         show.numbers=FALSE,levels=NA,clabel="",
-                         ellipse.col=c("#00FF0080","#FF000080"),
-                         type=1,ci.col='gray80',line.col='black',
-                         lwd=1,plot=TRUE,exterr=FALSE,model=1,
+isochron.UPb <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                         levels=NA,clabel="",
+                         ellipse.fill=c("#00FF0080","#FF000080"),
+                         ellipse.stroke='black',type=1,
+                         ci.col='gray80',line.col='black',lwd=1,
+                         plot=TRUE,exterr=FALSE,model=1,
                          show.ellipses=1*(model!=2),
-                         anchor=list(FALSE,NA),hide=NULL,omit=NULL,
-                         omit.col=NA,...){
+                         anchor=list(FALSE,NA),hide=NULL,
+                         omit=NULL,omit.fill=NA,omit.stroke='grey',...){
     ns <- length(x)
     plotit <- (1:ns)%ni%hide
     calcit <- (1:ns)%ni%c(hide,omit)
@@ -596,12 +600,13 @@ isochron.UPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
         out$y0['disp[y]'] <- sqrt(out$mswd)*out$y0['ci[y]']
     }
     if (plot){
-        scatterplot(XY,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.ellipses=show.ellipses,
+        scatterplot(XY,alpha=alpha,show.ellipses=show.ellipses,
                     show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=out,
                     ci.col=ci.col,line.col=line.col,lwd=lwd,
-                    hide=hide,omit=omit,omit.col=omit.col,...)
+                    hide=hide,omit=omit,omit.fill=omit.fill,
+                    omit.stroke=omit.stroke,...)
         graphics::title(isochrontitle(out,sigdig=sigdig,type='U-Pb'),
                         xlab=x.lab,ylab=y.lab)
     }
@@ -654,13 +659,14 @@ isochron.UPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
 #' @param growth add Stacey-Kramers Pb-evolution curve to the plot?
 #' @rdname isochron
 #' @export
-isochron.PbPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=TRUE,ci.col='gray80',
-                          line.col='black',lwd=1,plot=TRUE,
-                          exterr=TRUE,model=1,show.ellipses=1*(model!=2),
-                          growth=FALSE,hide=NULL,omit=NULL,omit.col=NA,...){
+isochron.PbPb <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=TRUE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE, exterr=TRUE,model=1,growth=FALSE,
+                          show.ellipses=1*(model!=2),hide=NULL,
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
     y <- data2york(x,inverse=inverse)
     d2calc <- clear(y,hide,omit)
     fit <- regression(d2calc,model=model)
@@ -687,12 +693,13 @@ isochron.PbPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
                                         exterr=exterr)[2]
     }
     if (plot) {
-        scatterplot(y,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.ellipses=show.ellipses,
+        scatterplot(y,alpha=alpha,show.ellipses=show.ellipses,
                     show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=out,
                     ci.col=ci.col,line.col=line.col,lwd=lwd,
-                    hide=hide,omit=omit,omit.col=omit.col,...)
+                    hide=hide,omit=omit,omit.fill=omit.fill,
+                    omit.stroke=omit.stroke,...)
         if (growth){
             xylim <- graphics::par('usr')
             if (xylim[1]<0) xylim[1] <- xylim[2]/100
@@ -717,13 +724,14 @@ isochron.PbPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
 }
 #' @rdname isochron
 #' @export
-isochron.ArAr <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=TRUE,ci.col='gray80',line.col='black',
-                          lwd=1,plot=TRUE,exterr=TRUE,model=1,
-                          show.ellipses=1*(model!=2),
-                          hide=NULL,omit=NULL,omit.col=NA,...){
+isochron.ArAr <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=TRUE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE,exterr=TRUE,model=1,
+                          show.ellipses=1*(model!=2),hide=NULL,
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
     y <- data2york(x,inverse=inverse)
     d2calc <- clear(y,hide,omit)
     fit <- regression(d2calc,model=model)
@@ -760,12 +768,13 @@ isochron.ArAr <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
                                   x$J[1],x$J[2],exterr=exterr)[2]
     }
     if (plot) {
-        scatterplot(y,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.ellipses=show.ellipses,
+        scatterplot(y,alpha=alpha,show.ellipses=show.ellipses,
                     show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=out,
                     ci.col=ci.col,line.col=line.col,lwd=lwd,
-                    hide=hide,omit=omit,omit.col=omit.col,...)
+                    hide=hide,omit=omit,omit.fill=omit.fill,
+                    omit.stroke=omit.stroke,...)
         graphics::title(isochrontitle(out,sigdig=sigdig,type='Ar-Ar'),
                         xlab=x.lab,ylab=y.lab)
     }
@@ -773,103 +782,117 @@ isochron.ArAr <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
 }
 #' @rdname isochron
 #' @export
-isochron.ThPb <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=FALSE,ci.col='gray80',line.col='black',
-                          lwd=1,plot=TRUE,exterr=TRUE,model=1,
-                          show.ellipses=1*(model!=2),
-                          hide=NULL,omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='Th232',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,
-                levels=levels,clabel=clabel,ellipse.col=ellipse.col,
-                inverse=inverse,ci.col=ci.col,line.col=line.col,
-                lwd=lwd,plot=plot,exterr=exterr,model=model,
-                show.ellipses=show.ellipses,hide=hide,omit=omit,
-                omit.col=omit.col,...)
-}
-#' @rdname isochron
-#' @export
-isochron.KCa <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                         show.numbers=FALSE,levels=NA,clabel="",
-                         ellipse.col=c("#00FF0080","#FF000080"),
-                         inverse=FALSE,ci.col='gray80',line.col='black',
-                         lwd=1,plot=TRUE,exterr=TRUE,model=1,
-                         show.ellipses=1*(model!=2),
-                         hide=NULL,omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='K40',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,
-                levels=levels,clabel=clabel,ellipse.col=ellipse.col,
-                inverse=inverse,ci.col=ci.col,line.col=line.col,
-                lwd=lwd,plot=plot,exterr=exterr,model=model,
-                show.ellipses=show.ellipses,bratio=0.895,
-                hide=hide,omit=omit,omit.col=omit.col,...)
-}
-#' @rdname isochron
-#' @export
-isochron.RbSr <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=FALSE,ci.col='gray80',line.col='black',
-                          lwd=1,plot=TRUE,exterr=TRUE,model=1,
+isochron.ThPb <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=FALSE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE,exterr=TRUE,model=1,
                           show.ellipses=1*(model!=2),hide=NULL,
-                          omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='Rb87',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,levels=levels,
-                clabel=clabel,ellipse.col=ellipse.col,inverse=inverse,
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='Th232',alpha=alpha,sigdig=sigdig,
+                show.numbers=show.numbers,levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
                 ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
                 exterr=exterr,model=model,show.ellipses=show.ellipses,
-                hide=hide,omit=omit,omit.col=omit.col,...)
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
 }
 #' @rdname isochron
 #' @export
-isochron.ReOs <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=FALSE,ci.col='gray80',line.col='black',
-                          lwd=1,plot=TRUE,exterr=TRUE,model=1,
-                          show.ellipses=1*(model!=2),hide=NULL,
-                          omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='Re187',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,
-                levels=levels,clabel=clabel,ellipse.col=ellipse.col,
-                inverse=inverse,ci.col=ci.col,line.col=line.col,lwd=lwd,
-                plot=plot,exterr=exterr,model=model,show.ellipses=show.ellipses,
-                hide=hide,omit=omit,omit.col=omit.col,...)
+isochron.KCa <- function(x,alpha=0.05,sigdig=2,show.numbers=FALSE,
+                         levels=NA,clabel="",inverse=FALSE,
+                         ci.col='gray80',
+                         ellipse.fill=c("#00FF0080","#FF000080"),
+                         ellipse.stroke='black',line.col='black',
+                         lwd=1, plot=TRUE,exterr=TRUE,model=1,
+                         show.ellipses=1*(model!=2),hide=NULL,
+                         omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='K40',alpha=alpha,sigdig=sigdig,
+                show.numbers=show.numbers,levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
+                ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
+                exterr=exterr,model=model,show.ellipses=show.ellipses,
+                bratio=0.895,hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
 }
 #' @rdname isochron
 #' @export
-isochron.SmNd <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
+isochron.RbSr <- function(x,alpha=0.05,sigdig=2,
                           show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=FALSE,ci.col='gray80',line.col='black',
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=FALSE,
+                          ci.col='gray80',line.col='black',
                           lwd=1,plot=TRUE,exterr=TRUE,model=1,
                           show.ellipses=1*(model!=2),hide=NULL,
-                          omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='Sm147',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,
-                levels=levels,clabel=clabel,ellipse.col=ellipse.col,
-                inverse=inverse,ci.col=ci.col,line.col=line.col,
-                lwd=lwd,plot=plot,exterr=exterr,model=model,
-                show.ellipses=show.ellipses,hide=hide,
-                omit=omit,omit.col=omit.col,...)
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='Rb87',alpha=alpha,sigdig=sigdig,
+                show.numbers=show.numbers,levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
+                ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
+                exterr=exterr,model=model,show.ellipses=show.ellipses,
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
 }
 #' @rdname isochron
 #' @export
-isochron.LuHf <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
-                          show.numbers=FALSE,levels=NA,clabel="",
-                          ellipse.col=c("#00FF0080","#FF000080"),
-                          inverse=FALSE,ci.col='gray80',line.col='black',
-                          lwd=1,plot=TRUE,exterr=TRUE,model=1,
+isochron.ReOs <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=FALSE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE,exterr=TRUE,model=1,
                           show.ellipses=1*(model!=2),hide=NULL,
-                          omit=NULL,omit.col=NA,...){
-    isochron_PD(x,nuclide='Lu176',xlim=xlim,ylim=ylim,alpha=alpha,
-                sigdig=sigdig,show.numbers=show.numbers,
-                levels=levels,clabel=clabel,ellipse.col=ellipse.col,
-                inverse=inverse,ci.col=ci.col,line.col=line.col,
-                lwd=lwd,plot=plot,exterr=exterr,model=model,
-                show.ellipses=show.ellipses,hide=hide,
-                omit=omit,omit.col=omit.col,...)
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='Re187',alpha=alpha,sigdig=sigdig,
+                show.numbers=show.numbers,levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
+                ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
+                exterr=exterr,model=model,show.ellipses=show.ellipses,
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
+}
+#' @rdname isochron
+#' @export
+isochron.SmNd <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=FALSE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE,exterr=TRUE,model=1,
+                          show.ellipses=1*(model!=2),hide=NULL,
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='Sm147',alpha=alpha,sigdig=sigdig,
+                show.numbers=show.numbers, levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
+                ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
+                exterr=exterr,model=model,show.ellipses=show.ellipses,
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
+}
+#' @rdname isochron
+#' @export
+isochron.LuHf <- function(x,alpha=0.05,sigdig=2, show.numbers=FALSE,
+                          levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',inverse=FALSE,
+                          ci.col='gray80',line.col='black',lwd=1,
+                          plot=TRUE,exterr=TRUE,model=1,
+                          show.ellipses=1*(model!=2),hide=NULL,
+                          omit=NULL,omit.fill=NA,omit.stroke='grey',...){
+    isochron_PD(x,nuclide='Lu176',alpha=alpha, sigdig=sigdig,
+                show.numbers=show.numbers, levels=levels,
+                clabel=clabel,ellipse.fill=ellipse.fill,
+                ellipse.stroke=ellipse.stroke,inverse=inverse,
+                ci.col=ci.col,line.col=line.col,lwd=lwd,plot=plot,
+                exterr=exterr,model=model,show.ellipses=show.ellipses,
+                hide=hide,omit=omit,omit.fill=omit.fill,
+                omit.stroke=omit.stroke,...)
 }
 #' @param type following the classification of
 #' Ludwig and Titterington (1994), one of either:
@@ -887,13 +910,14 @@ isochron.LuHf <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
 #' vs. \eqn{^{232}}Th/\eqn{^{238}}U
 #' @rdname isochron
 #' @export
-isochron.ThU <- function (x,type=2,xlim=NA,ylim=NA,alpha=0.05,
-                          sigdig=2,show.numbers=FALSE,levels=NA,
-                          clabel="",ellipse.col=c("#00FF0080","#FF000080"),
-                          ci.col='gray80',line.col='black',lwd=1,
-                          plot=TRUE,exterr=TRUE,model=1,
-                          show.ellipses=1*(model!=2),hide=NULL,
-                          omit=NULL,omit.col=NA,...){
+isochron.ThU <- function (x,type=2,alpha=0.05,sigdig=2,
+                          show.numbers=FALSE,levels=NA,clabel="",
+                          ellipse.fill=c("#00FF0080","#FF000080"),
+                          ellipse.stroke='black',ci.col='gray80',
+                          line.col='black',lwd=1,plot=TRUE,
+                          exterr=TRUE,model=1,show.ellipses=1*(model!=2),
+                          hide=NULL,omit=NULL,omit.fill=NA,
+                          omit.stroke='grey',...){
     if (x$format %in% c(1,2)){
         out <- isochron_ThU_3D(x,type=type,model=model,
                                exterr=exterr,alpha=alpha,
@@ -911,12 +935,12 @@ isochron.ThU <- function (x,type=2,xlim=NA,ylim=NA,alpha=0.05,
         out$displabel <- quote('('^234*'U/'^238*'U)-dispersion = ')
     }
     if (plot){
-        scatterplot(out$xyz,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
-                    show.ellipses=show.ellipses,ci.col=ci.col,
-                    line.col=line.col,lwd=lwd,
-                    hide=hide,omit=omit,omit.col=omit.col,...)
+        scatterplot(out$xyz,alpha=alpha,show.numbers=show.numbers,
+                    levels=levels,clabel=clabel,
+                    ellipse.fill=ellipse.fill,ellipse.stroke=ellipse.stroke,
+                    fit=out,show.ellipses=show.ellipses,ci.col=ci.col,
+                    line.col=line.col,lwd=lwd,hide=hide,omit=omit,
+                    omit.fill=omit.fill,omit.stroke=omit.stroke,...)
         graphics::title(isochrontitle(out,sigdig=sigdig,
                         type=intercept.type,units='ka'),
                         xlab=out$xlab,ylab=out$ylab)
@@ -925,12 +949,13 @@ isochron.ThU <- function (x,type=2,xlim=NA,ylim=NA,alpha=0.05,
 }
 #' @rdname isochron
 #' @export
-isochron.UThHe <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
+isochron.UThHe <- function(x,alpha=0.05,sigdig=2,
                            show.numbers=FALSE,levels=NA,clabel="",
-                           ellipse.col=c("#00FF0080","#FF000080"),
-                           ci.col='gray80',line.col='black',lwd=1,
-                           plot=TRUE,model=1,show.ellipses=2*(model!=2),
-                           hide=NULL,omit=NULL,omit.col='grey',...){
+                           ellipse.fill=c("#00FF0080","#FF000080"),
+                           ellipse.stroke='black',ci.col='gray80',
+                           line.col='black',lwd=1,plot=TRUE,model=1,
+                           show.ellipses=2*(model!=2),hide=NULL,
+                           omit=NULL,omit.fill=NA,omit.stroke='grey',...){
     y <- data2york(x)
     d2calc <- clear(y,hide,omit)
     fit <- regression(d2calc,model=model)
@@ -943,12 +968,13 @@ isochron.UThHe <- function(x,xlim=NA,ylim=NA,alpha=0.05,sigdig=2,
     out$displabel <- quote('He-dispersion = ')
     out$y0label <- quote('He'[o]*' = ')
     if (plot) {
-        scatterplot(y,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
+        scatterplot(y,alpha=alpha,show.numbers=show.numbers,levels=levels,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=out,
                     show.ellipses=show.ellipses,ci.col=ci.col,
                     line.col=line.col,lwd=lwd,hide=hide,
-                    omit=omit,omit.col=omit.col,...)
+                    omit=omit,omit.fill=omit.fill,
+                    omit.stroke=omit.stroke,...)
         graphics::title(isochrontitle(out,sigdig=sigdig,type='U-Th-He'),
                         xlab="P",ylab="He")
     }
@@ -1064,11 +1090,12 @@ isochron_ThU_2D <- function(x,type=2,model=1,exterr=TRUE,
     out
 }
 
-isochron_PD <- function(x,nuclide,xlim=NA,ylim=NA,alpha=0.05,
-                        sigdig=2,show.numbers=FALSE,levels=NA,
-                        clabel="",ellipse.col=c("#00FF0080","#FF000080"),
-                        inverse=FALSE,ci.col='gray80',line.col='black',
-                        lwd=1,plot=TRUE,exterr=TRUE,model=1,
+isochron_PD <- function(x,nuclide,alpha=0.05,sigdig=2,
+                        show.numbers=FALSE,levels=NA, clabel="",
+                        ellipse.fill=c("#00FF0080","#FF000080"),
+                        ellipse.stroke='black',inverse=FALSE,
+                        ci.col='gray80',line.col='black',lwd=1,
+                        plot=TRUE,exterr=TRUE,model=1,
                         show.ellipses=1*(model!=2),bratio=1,
                         hide=NULL,omit=NULL,...){
     y <- data2york(x,inverse=inverse)
@@ -1097,10 +1124,10 @@ isochron_PD <- function(x,nuclide,xlim=NA,ylim=NA,alpha=0.05,
     out$y0label <-
         substitute(a*b*c,list(a='(',b=lab$y,c=quote(')'[o]*' = ')))
     if (plot){
-        scatterplot(y,xlim=xlim,ylim=ylim,alpha=alpha,
-                    show.ellipses=show.ellipses,
+        scatterplot(y,alpha=alpha,show.ellipses=show.ellipses,
                     show.numbers=show.numbers,levels=levels,
-                    clabel=clabel,ellipse.col=ellipse.col,fit=out,
+                    clabel=clabel,ellipse.fill=ellipse.fill,
+                    ellipse.stroke=ellipse.stroke,fit=out,
                     ci.col=ci.col,line.col=line.col,lwd=lwd,
                     hide=hide,omit=omit,...)
         graphics::title(isochrontitle(out,sigdig=sigdig,type='PD'),
